package CoBlog::Controller::Admin::Article;

use strict;
use warnings;
use base 'Catalyst::Controller';
use FormElementContainer;

=head1 NAME

CoBlog::Controller::Admin::Article - Catalyst Controller

=head1 DESCRIPTION

Catalyst Controller.

=head1 METHODS

=cut


=head2 index 

=cut

sub index : Private {
    my ( $self, $c ) = @_;

    $c->forward('list');
}

=head2 list

List of all articles

=cut

sub list : Local {
    my ( $self, $c, $return_count ) = @_;
    
    $c->stash->{articles} = [ sort { $b->updated cmp $a->updated } $c->model('CoBlogDB::Article')->all ];
    $c->stash->{template}='admin/article/list.tt2';
}

=head2 make_article_widget

Build an HTML::Widget form for article creation and updates

=cut

sub make_article_widget {
    my ($self, $c) = @_;

    # Create an HTML::Widget to build the form
    my $w = $c->widget('article_form')->method('post');

    # Use custom class to render each element in the form    
    $w->element_container_class('FormElementContainer');

    # Create the form feilds
    $w->element('Hidden',    'id'     )->value("");
    $w->element('Textfield', 'title'  )->label('Title');
    $w->element('Textarea',  'content' )->label('Content');
    $w->element('Textfield', 'keyword_list' )->label('Keywords');
    $w->element('Submit',    'submit' )->value('submit');

    # Set constraints
    $w->constraint(All     => qw/title content/)
        ->message('Required. ');
    $w->constraint(Length  => qw/title content/)->min(10)
        ->message('Must be minimum 10 characters. ');

    # Set filters
    for my $column (qw/title content keyword_list/) {
        $w->filter( HTMLEscape => $column );
        $w->filter( TrimEdges  => $column );
    }

    # Return the widget    
    return $w;
}

=head2 edit

Build an HTML::Widget form for article updates

=cut

sub edit : Local {
    my ($self, $c, $article_id ) = @_;

    # Create the widget and set the action for the form
    my $w = $self->make_article_widget($c);
    $w->action($c->uri_for('create_do'));

    if ( my $article=$c->model('CoBlogDB::Article')->find({ id => $article_id}) ) {
        $article->fill_widget( $w );

        # Fetch all keywords currently in use
        my %keywords;
        my $keyword_rs=$c->model('CoBlogDB::ArticleKeywords')->search({ article_id => $article_id });
        while ( my $keyword = $keyword_rs->next ) {
            $keywords{ $keyword->keyword } = $keyword->keyword;
        }
        (my $keyword_element)=$w->get_elements( name => 'keyword_list' );
        $keyword_element->value( join(', ', sort keys %keywords) );
        
        # Write form to stash variable for use in template
        $c->stash->{widget_result} = $w->result;

        # Set the template
        $c->stash->{template} = 'admin/article/create.tt2';
    }
    else {
        # Notify user and return to index page
        $c->stash->{error_msg} = 'Article could not be found!';
        $c->forward('index');
    }
    
}

=head2 create

Build an HTML::Widget form for article creation

=cut

sub create : Local {
    my ($self, $c) = @_;

    # Create the widget and set the action for the form
    my $w = $self->make_article_widget($c);
    $w->action($c->uri_for('create_do'));

    # Write form to stash variable for use in template
    $c->stash->{widget_result} = $w->result;

    # Set the template
    $c->stash->{template} = 'admin/article/create.tt2';
}

=head2 create_do

Stores the new or updated object in the database, after validation

=cut

sub create_do : Local {
    my ($self, $c) = @_;

    # Create the widget and set the action for the form
    my $w = $self->make_article_widget($c);
    $w->action($c->uri_for('create_do'));

    # Validate the form parameters
    my $result = $w->process($c->req);

    # Write form (including validation error messages) to
    # stash variable for use in template
    $c->stash->{widget_result} = $result;

    # Were their validation errors?
    if ($result->has_errors) {
        # Warn the user at the top of the form that there were errors.
        # Note that there will also be per-field feedback on
        # validation errors because of '$w->process($c->req)' above.
        $c->stash->{error_msg} = 'Errors in form. Please verify your information!';

        # Set the template
        $c->stash->{template} = 'admin/article/create.tt2';

    } else {
        
        if ( $result->params->{'id'} ) {

            # Fetch existing object
            my $article = $c->model('CoBlogDB::Article')->find({ id => $result->params->{'id'} });

            # Assign author (currently logged in user)
            my $author=$c->model('CoBlogDB::Author')->find({ username => $c->user->username });
            $article->updated_by($author->id);
            $article->populate_from_widget($result);
            
            # Wipe old keywords
            $article->delete_related('keywords');

            # Assign new list of keywords
            my $keywords = $result->params->{'keyword_list'};
            my @keywords=split(/,/, $keywords);
            foreach my $keyword (@keywords) {
                $keyword=~s/^\s*(.*?)\s*$/$1/;
                $article->update_or_create_related('keywords', { keyword => $keyword });
            }

            # Set a status message for the user
            $c->flash->{status_msg} = 'Article updated.';

        }
        else {
            
            # Create new object
            my $article = $c->model('CoBlogDB::Article')->new({});
            
            # Assign author (currently logged in user)
            my $author=$c->model('CoBlogDB::Author')->find({ username => $c->user->username });
            $article->updated_by($author->id);

            # Store data from widget and fetch the primary key ID
            # from the RS storage handler.
            $article=$article->populate_from_widget($result);
            $article->id( $article->result_source->storage->last_insert_id() );
            
            # Assign keywords
            my $keywords = $result->params->{'keyword_list'};
            my @keywords=split(/,/, $keywords);
            foreach my $keyword (@keywords) {
                $keyword=~s/^\s*(.*?)\s*$/$1/;
                $article->create_related('keywords', { keyword => $keyword });
            }

            # Set a status message for the user
            $c->flash->{status_msg} = 'Article created.';

        }
        
        # Redisplay an empty form for another
        $c->stash->{widget_result} = $w->result;

        # Redirect back to list
        $c->response->redirect($c->uri_for('list'));

    }

}

=head2 delete 

Delete an article
    
=cut

sub delete : Local {
    # $id = primary key of book to delete
    my ($self, $c, $id) = @_;

    # Search for the article and then delete it
    # (using delete_all will also remove all keywords assosciated with this article).
    $c->model('CoBlogDB::Article')->search({id => $id})->delete_all;

    # Set a status message to be displayed at the top of the view
    $c->flash->{status_msg} = "Article deleted.";

    $c->response->redirect($c->uri_for('list'));
}

=head1 AUTHOR

Robin Smidsrød,,,

=head1 LICENSE

This library is free software, you can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

1;
